/*******************************************************************************
 * Copyright (c) 2023, Jan Koester jan.koester@gmx.net
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * Neither the name of the <organization> nor the
 *      names of its contributors may be used to endorse or promote products
 *      derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *******************************************************************************/

#include <htmlpp/html.h>
#include <httppp/http.h>
#include <httppp/exception.h>

#include <database.h>
#include <auth.h>
#include <conf.h>
#include <plugin.h>

namespace blogi {
    class Content : public PluginApi{
    public:
        Content(){
        };

        ~Content(){
        };

        void contentIdxPage(netplus::con *curcon,libhttppp::HttpRequest *curreq,const char *tag){
            libhttppp::HTTPException excep;
            libhtmlpp::HtmlString condat;

            std::string sid;
            blogi::SQL sql;
            blogi::DBResult res;

            int ncount=0;

            bool tagsavble=true;

            if (tag) {
                sql = "select id from tags where name='"; sql << tag << "' LIMIT 1";
                if(Args->database->exec(&sql,res)<1){
                    excep[libhttppp::HTTPException::Critical] << "no tag data found for this name!";
                    throw excep;
                }else {
                    sql = "select content.id,content.title,content.descrition,users.username,content.created from content ";
                    sql <<"LEFT JOIN users ON content.author=users.id LEFT JOIN tags_content ON tags_content.content_id=content.id where tags_content.tag_id='"
                    << res[0][0]
                    <<"' ORDER BY id DESC";
                    ncount=Args->database->exec(&sql,res);
                }
            } else {
                sql="select content.id,content.title,content.descrition,users.username,content.created from content LEFT JOIN users ON content.author=users.id ORDER BY id DESC";
                ncount=Args->database->exec(&sql,res);
            }

            if (ncount<1) {
                excep[libhttppp::HTTPException::Critical] << "<span> No Content found! </span>";
                throw excep;
            }

            condat << "<div id=\"contentidx\">";
            if(Args->auth->isLoggedIn(curreq,sid)){
                condat << "<div class=\"blog_adminmenu\">"
                << "<ul>"
                << "<li><a href=\""<< BUILDURL(content/addpost) <<"\">Addpost</a></li>"
                << "</ul>"
                << "</div>";
            }
            for (int i = 0; i < ncount; i++) {
                condat << "<div class=\"blog_entry\">"
                << "<span class=\"title\">" << res[i][1] << "</span>"
                << "<div  class=\"entry_text\">" << res[i][2] << "</div>"
                << "<span><a href=\""<< BUILDURL(content/read/) << res[i][0] << "\">Weiterlesen</a> </span>"
                << "<span class=\"author\">verfasst von " << res[i][3] << " am "<< res[i][4]  <<" </span></div>";
            }

            condat << "</div>";

            sql="SELECT name,id FROM tags";

            int tcount=Args->database->exec(&sql,res);


            if (tcount>0) {
                condat << "<div id=\"idxtags\"><ul>";

                for (int i = 0; i < tcount; i++) {
                    sql="select from tags_content where tag_id = '";
                    sql << res[i][1] << "'";
                    blogi::DBResult rescnt;
                    condat << "<li><a href=\""
                    << BUILDURL(content/tag/) << res[i][0] << "\">" << res[i][0] << "(" << Args->database->exec(&sql,rescnt) << ")" << "</a></li>";

                }

                condat << "</ul></div>";
            }
            std::string out;
            Args->theme->RenderSite(out,curreq->getRequestURL(),condat,Args->auth->isLoggedIn(curreq,sid));

            libhttppp::HttpResponse resp;
            resp.setVersion(HTTPVERSION(1.1));
            resp.setContentType(HTTP200);
            resp.setContentType("text/html");
            resp.send(curcon,out.c_str(),out.length());
        };

        void contentPage(netplus::con* curcon, libhttppp::HttpRequest* curreq) {
            libhttppp::HTTPException excep;
            libhtmlpp::HtmlString condat;

            std::string sid;
            int cid;

            sscanf(curreq->getRequestURL(), BUILDURL(content/read/%d),&cid);

            blogi::SQL sql;

            sql << "select content.id,content.title,content.text,users.username,content.created from content LEFT JOIN users ON content.author=users.id WHERE content.id='" << cid <<"' LIMIT 1";

            blogi::DBResult res;


            if(Args->database->exec(&sql,res)<1){
                excep[libhttppp::HTTPException::Critical] << "No entry found for content id: " << cid;
                throw excep;
            }

            condat << "<div id=\"content\">";
            if (Args->auth->isLoggedIn(curreq,sid)) {
                condat << "<div class=\"blog_adminmenu\">"
                << "<ul>"
                << "<li><a href=\"" << BUILDURL(content/edit/) << cid << "\">Bearbeiten</a></li>"
                << "<li><a href=\"" << BUILDURL(content/del/) << cid << "\">Loeschen</a></li>"
                << "</ul>"
                << "</div>";
            }
            condat << "<div class=\"blog_entry\">"
            << "<span class=\"title\">" << res[0][1] << "</span>"
            << "<div  class=\"text\">" << res[0][2] << "</div>"
            << "<span class=\"author\">verfasst von " << res[0][3] << " am "<< res[0][4]  <<" </span>"
            << "</div></div>";

            std::string out;
            Args->theme->RenderSite(out,curreq->getRequestURL(),condat,Args->auth->isLoggedIn(curreq,sid));

            libhttppp::HttpResponse resp;
            resp.setVersion(HTTPVERSION(1.1));
            resp.setContentType(HTTP200);
            resp.setContentType("text/html");
            resp.send(curcon,out.c_str(),out.length());
        };

        void addPostPage(netplus::con *curcon,libhttppp::HttpRequest *curreq){
            std::string sid;
            if(!Args->auth->isLoggedIn(curreq,sid)){
                libhttppp::HTTPException exp;
                exp[libhttppp::HTTPException::Error] << "Please login before!";
                throw exp;
            }

            libhtmlpp::HtmlString condat;
            libhttppp::HTTPException excep;

            condat << "<div id=\"content\">"
            << "<form action=\""<< BUILDURL(content/addpost) << "\" method=\"post\" enctype=\"multipart/form-data\" >"
            << "<span>Titel:</span><input maxlength=\"255\" name=\"blog_title\" type=\"text\" style=\"width: 100%;\" /><br/>"
            << "<span>Kurzbeschreibung:</span><input maxlength=\"255\" name=\"blog_descrition\" type=\"text\"  style=\"width: 100%;\" /><br/>"
            << "<span>Schlagworte:</span><input name=\"blog_tags\" type=\"text\"  style=\"width: 100%;\" /><br/>"
            << "<span>Text:</span><textarea name=\"blog_content\" type=\"text\" style=\"width: 100%; height:480px;\"> </textarea>"
            << "<input type=\"submit\" value=\"Blog eintrag Posten\"/>"
            << "</form>"
            << "</div>";

            libhttppp::HttpForm curform;
            curform.parse(curreq);

            std::string title;
            std::string text;
            std::string tags;
            std::string descrition;

            if(curform.getBoundary()){
                for(libhttppp::HttpForm::MultipartFormData *curformdat=curform.getMultipartFormData(); curformdat; curformdat=curformdat->nextMultipartFormData()){
                    libhttppp::HttpForm::MultipartFormData::ContentDisposition *curctdisp=curformdat->getContentDisposition();
                    if(curformdat->getContentType())
                        condat << "ContentType: " << curformdat->getContentType() << "<br>\r\n";
                    if(curctdisp->getName()){
                        if (strcmp(curctdisp->getName(), "blog_title") == 0) {
                            title.append(curformdat->getData(),curformdat->getDataSize());
                        }else if(strcmp(curctdisp->getName(), "blog_descrition") == 0){
                            descrition.append(curformdat->getData(), curformdat->getDataSize());
                        }else if (strcmp(curctdisp->getName(), "blog_tags") == 0) {
                            tags.append(curformdat->getData(), curformdat->getDataSize());
                        }
                        else if(strcmp(curctdisp->getName(),"blog_content")==0){
                            text.append(curformdat->getData(), curformdat->getDataSize());;
                        }
                    }
                }

                size_t content_id = -1;

                if(!text.empty() && !title.empty() && !descrition.empty()){

                    blogi::SQL asql,sql;
                    blogi::DBResult ares,res;

                    std::string uid;
                    Args->session->getSessionData(sid.c_str(),"uid",uid);

                    asql << "SELECT id from users WHERE sid='"<< uid.c_str() <<"' LIMIT 1;";

                    if (Args->database->exec(&asql,ares) < 1) {
                        excep[libhttppp::HTTPException::Error] << "User with SID not found!";
                        throw excep;
                    }

                    int author=atoi(ares[0][0]);


                    time_t t = time(NULL);
                    struct tm time = { 0 };
                    char ttmp[26];
                    localtime_r(&t,&time);
                    asctime_r(&time,ttmp);

                    sql << "INSERT INTO content (title,descrition,text,author,created) VALUES ('"
                    << title.c_str() << "','" << descrition.c_str() <<"','"<< text.c_str()
                    <<"','"<< author <<"','" << ttmp << "') RETURNING id;";

                    try {
                        Args->database->exec(&sql,res);
                    }catch(libhttppp::HTTPException &e){
                        throw e;
                    }
                    content_id=atoi(res[0][0]);

                }

                size_t tgsstart=0,tgsend=0;
                if (!tags.empty() && content_id != -1) {
                    for (int i = 0; i <= tags.length(); ++i) {
                        if (tags[i] == ' ' || i == tags.length()) {
                            if((i - tgsstart) >0 ){
                                std::string tag= tags.substr(tgsstart,(i-tgsstart));
                                tgsstart = i+1;
                                int tries = 0;
                                TAGNAMECHECK:
                                blogi::SQL sql;
                                blogi::DBResult res;
                                if (tries>5) {
                                    excep[libhttppp::HTTPException::Critical] << "tag doesn't exists and could not created!";
                                    throw excep;
                                }

                                sql = "select id,name from tags where name='"; sql << tag.c_str() << "' LIMIT 1;";

                                if (Args->database->exec(&sql,res) != 1) {
                                    sql = "insert into tags (name) VALUES ('"; sql << tag.c_str() <<"');";
                                    Args->database->exec(&sql,res);
                                    ++tries;
                                    goto TAGNAMECHECK;
                                }

                                sql =  "insert into tags_content (content_id,tag_id) VALUES ('"; sql << content_id <<"','" << res[0][0] <<"');";
                                Args->database->exec(&sql,res);
                            }
                        }
                    }
                }
            }
            std::string out;
            Args->theme->RenderSite(out,curreq->getRequestURL(),condat,Args->auth->isLoggedIn(curreq,sid));

            libhttppp::HttpResponse resp;
            resp.setVersion(HTTPVERSION(1.1));
            resp.setContentType(HTTP200);
            resp.setContentType("text/html");
            resp.send(curcon,out.c_str(),out.length());
        }

        void delPostPage(netplus::con* curcon, libhttppp::HttpRequest* curreq) {
            std::string sid;
            if(!Args->auth->isLoggedIn(curreq,sid)){
                libhttppp::HTTPException exp;
                exp[libhttppp::HTTPException::Error] << "Please login before!";
                throw exp;
            }

            libhttppp::HTTPException excep;

            blogi::DBResult res;
            blogi::SQL      sql;

            int cid = -1;

            sscanf(curreq->getRequestURL(), BUILDURL(content/del/%d), &cid);

            sql ="DELETE FROM tags_content WHERE content_id='";
            sql << cid << "'; " <<"DELETE FROM content WHERE id='"; sql << cid << "';";

            Args->database->exec(&sql,res);

            cleartags();

            libhtmlpp::HtmlString condat;

            condat << "<span>Beitrag geloescht</span>";

            std::string out;
            Args->theme->RenderSite(out,curreq->getRequestURL(),condat,Args->auth->isLoggedIn(curreq,sid));

            libhttppp::HttpResponse resp;
            resp.setVersion(HTTPVERSION(1.1));
            resp.setContentType(HTTP200);
            resp.setContentType("text/html");
            resp.send(curcon,out.c_str(),out.length());

        }

        void cleartags() {

            blogi::SQL sql;
            blogi::DBResult res;

            sql="SELECT name,id FROM tags";

            int n=Args->database->exec(&sql,res);

            if(n<0){
                return;
            }

            libhttppp::HTTPException excep;


            for (int i = 0; i < n; i++) {

                sql="SELECT COUNT(*) FROM tags_content WHERE tag_id='"; sql << res[i][1] << "';";

                blogi::DBResult res2;

                if(Args->database->exec(&sql,res2)<1){
                    libhttppp::HTTPException excep;
                    excep[libhttppp::HTTPException::Critical] << "clear tags count failed this shouldn't happend!";
                    throw excep;
                }

                if( atoi(res2[0][0])>0)
                    continue;

                sql = "DELETE FROM tags WHERE id='"; sql << res[i][1] << "';";

                Args->database->exec(&sql,res2);

            }
        }

        void editPostPage(netplus::con* curcon, libhttppp::HttpRequest* curreq) {/*
            if (!isLoggedIn(curreq)) {
                loginPage(curcon, curreq);
                return;
        }

        libhttppp::HTTPException excep;

        int cid = -1;

        sscanf(curreq->getRequestURL(), BUILDURL(content/edit/%d), &cid);

        char sql[255];

        libhttppp::HttpForm curform;
        curform.parse(curreq);

        std::string title;
        std::string text;
        std::string tags;
        std::string descrition;

        if (curform.getBoundary()) {
            for (libhttppp::HttpForm::MultipartFormData* curformdat = curform.getMultipartFormData(); curformdat; curformdat = curformdat->nextMultipartFormData()) {
                libhttppp::HttpForm::MultipartFormData::ContentDisposition* curctdisp = curformdat->getContentDisposition();
                if (curctdisp->getName()) {
                    if (strcmp(curctdisp->getName(), "blog_title") == 0) {
                        title.append(curformdat->getData(), curformdat->getDataSize());
        }
        else if (strcmp(curctdisp->getName(), "blog_descrition") == 0) {
            descrition.append(curformdat->getData(), curformdat->getDataSize());
        }
        else if (strcmp(curctdisp->getName(), "blog_tags") == 0) {
            tags.append(curformdat->getData(), curformdat->getDataSize());
        }
        else if (strcmp(curctdisp->getName(), "blog_content") == 0) {
            text.append(curformdat->getData(), curformdat->getDataSize());;
        }
        }
        }

        if (!text.empty() && !title.empty() && !descrition.empty()) {

            size_t sqltextsize = ((title.length() + text.length()) + 255);
            char sqltext[sqltextsize];

            snprintf(sqltext, sqltextsize, "UPDATE content SET title='%s',descrition='%s',text='%s' where id=%d;",
            title.c_str(), descrition.c_str(), text.c_str(),cid);

            PGresult* textres = PQexec(conn, sqltext);

            if (PQresultStatus(textres) != PGRES_COMMAND_OK) {
                excep[libhttppp::HTTPException::Critical] << PQerrorMessage(conn);
                PQclear(textres);
                throw excep;
        }

        PQclear(textres);
        }

        snprintf(sql, 255, "DELETE FROM tags_content WHERE content_id='%d';", cid);
        PGresult* delctags = PQexec(conn, sql);

        if (PQresultStatus(delctags) != PGRES_COMMAND_OK) {
            excep[libhttppp::HTTPException::Critical] << PQerrorMessage(conn);
            PQclear(delctags);
            throw excep;
        }


        PQclear(delctags);

        cleartags();

        size_t tgsstart = 0, tgsend = 0;
        if (!tags.empty() && cid != -1) {
            for (int i = 0; i <= tags.length(); ++i) {
                if (tags[i] == ' ' || i == tags.length()) {
                    if ((i - tgsstart) > 0) {
                        std::string tag = tags.substr(tgsstart, (i - tgsstart));
                        tgsstart = i + 1;
                        int tries = 0;

                        TAGNAMECHECK:
                        if (tries > 5) {
                            excep[libhttppp::HTTPException::Critical] << "tag doesn't exists and could not created!";
                            throw excep;
        }
        snprintf(sql, 255, "select id,name from tags where name='%s' LIMIT 1;", tag.c_str());
        PGresult* tagres = PQexec(conn, sql);

        if (PQresultStatus(tagres) != PGRES_TUPLES_OK) {
            excep[libhttppp::HTTPException::Critical] << PQerrorMessage(conn);
            PQclear(tagres);
            throw excep;
        }

        if (PQntuples(tagres) != 1) {
            PQclear(tagres);
            snprintf(sql, 255, "insert into tags (name) VALUES ('%s');", tag.c_str());
            tagres = PQexec(conn, sql);
            PQclear(tagres);
            ++tries;
            goto TAGNAMECHECK;
        }

        snprintf(sql, 255, "insert into tags_content (content_id,tag_id) VALUES ('%d','%s');", cid, PQgetvalue(tagres, 0, 0));
        PQclear(tagres);
        tagres = PQexec(conn, sql);

        if (PQresultStatus(tagres) != PGRES_COMMAND_OK) {
            excep[libhttppp::HTTPException::Critical] << PQerrorMessage(conn);
            PQclear(tagres);
            throw excep;
        }

        PQclear(tagres);
        }
        }
        }
        }
        }

        snprintf(sql, 255, "SELECT title,descrition,text,tags.name FROM content \
LEFT JOIN tags_content ON tags_content.content_id = content.id \
LEFT JOIN tags ON tags.id = tags_content.tag_id where content.id = '%d';", cid);

PGresult* ctxtres = PQexec(conn, sql);
libhtmlpp::HtmlString condat;

if (PQresultStatus(ctxtres) != PGRES_TUPLES_OK) {
    excep[libhttppp::HTTPException::Critical] << PQerrorMessage(conn);
    PQclear(ctxtres);
    throw excep;
        }

        if (PQntuples(ctxtres) > 0) {
            condat << "<div id=\"content\">"
            << "<form action=\"" << BUILDURL(content/edit/ ) << cid << "\" method=\"post\" enctype=\"multipart/form-data\" >"
            << "<span>Titel:</span><input maxlength=\"255\" name=\"blog_title\" type=\"text\" style=\"width: 100%;\" value=\"" << PQgetvalue(ctxtres, 0, 0) << "\"/><br/>"
            << "<span>Kurzbeschreibung:</span><input maxlength=\"255\" name=\"blog_descrition\" type=\"text\"  style=\"width: 100%;\" value=\"" << PQgetvalue(ctxtres, 0, 1) << "\"/><br/>"
            << "<span>Schlagworte:</span><input name=\"blog_tags\" type=\"text\"  style=\"width: 100%;\" value=\"";
            for (int i = 0; i < PQntuples(ctxtres); ++i) {
                condat << PQgetvalue(ctxtres, i, 3);
                if (PQntuples(ctxtres) - 1 != i)
                    condat << " ";
        }
        condat << "\"/><br/>"
        << "<span>Text:</span><textarea name=\"blog_content\" type=\"text\" style=\"width: 100%; height:480px;\" textarea>" << PQgetvalue(ctxtres, 0, 2) << "</textarea>"
        << "<input type=\"submit\" value=\"Blog eintrag Posten\"/>"
        << "</form>"
        << "</div>";
        }

        PQclear(ctxtres);

        indexPage(curcon, curreq, condat);*/
        }

        const char *getName(){
            return "content";
        }

        const char *getVersion(){
            return "0.2alpha";
        }

        const char *getAuthor(){
            return "Jan Koester";
        }

        void initPlugin(){
            return;
        }

        bool Controller(netplus::con * curcon, libhttppp::HttpRequest * req){
            if(strncmp(req->getRequestURL(),BUILDURL(content),strlen(BUILDURL(content)))!=0){
                return false;
            }

            if (strncmp(req->getRequestURL(),BUILDURL(content/addpost),strlen(BUILDURL(content/addpost)))==0){
                    addPostPage(curcon,req);
            }else if (strncmp(req->getRequestURL(), BUILDURL(content/read), strlen(BUILDURL(content/read))) == 0) {
                    contentPage(curcon,req);
            }else if (strncmp(req->getRequestURL(), BUILDURL(content/edit), strlen(BUILDURL(content/edit))) == 0) {
                    editPostPage(curcon,req);
            }else if (strncmp(req->getRequestURL(), BUILDURL(content/del), strlen(BUILDURL(content/del))) == 0) {
                    delPostPage(curcon, req);
            }else{
                if (strcmp(req->getRequestURL(),BUILDURL(content/tag))>0){
                    size_t len = strlen(BUILDURL(content/tag/));
                    contentIdxPage(curcon,req,req->getRequestURL()+len);
                 }else{
                    contentIdxPage(curcon,req,nullptr);
                 }
            }
            return true;
        }
    };
};

extern "C" blogi::PluginApi* create() {
    return new blogi::Content();
}

extern "C" void destroy(blogi::PluginApi* p) {
    delete p;
}

class __attribute__((weak)) Content;
