/*******************************************************************************
Copyright (c) 2021, Jan Koester jan.koester@gmx.net
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of the <organization> nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*******************************************************************************/

#include <iostream>

#include <signal.h>
#include <string.h>
#include <errno.h>

#include <netplus/eventapi.h>
#include <cryptplus/cryptplus.h>

#include <httppp/exception.h>
#include <httppp/http.h>
#include <httppp/httpd.h>

#include <htmlpp/html.h>
#include <htmlpp/exception.h>

#include "session.h"
#include "auth.h"
#include "database.h"
#include "conf.h"

#include "blogi.h"
#include "pgsql.cpp"


/*
class Controller : public netplus::event {
public:


    
    void settingsPage(netplus::con *curcon,libhttppp::HttpRequest *curreq){
        std::string sessid;
        if(!_BlogAuth->isLoggedIn(curreq,_session,sessid)){
            loginPage(curcon,curreq);
            return;
        }
        libhtmlpp::HtmlString condat;
        libhttppp::HTTPException excep;
        
        condat << "<div id=\"content\">"
               << "<span>Settings:</span>"
               << "</div>";
        
        indexPage(curcon,curreq,condat);    
    }

};*/

blogi::Blogi::Blogi(netplus::socket *serversocket) : event(serversocket){

    PlgArgs = new PluginArgs;
    PlgArgs->database= new Postgresql(POSTGRES_DB);
    PlgArgs->session= new Session();
    PlgArgs->auth=new Auth(PlgArgs->database,PlgArgs->session);

    TemplateConfig tplcfg;
    tplcfg.Path=TEMPLATEPATH;
    tplcfg.Theme=TEMPLATENAME;
    tplcfg.TDatabase=PlgArgs->database;

    PlgArgs->theme=new Template(tplcfg);

    BlogiPlg = new blogi::Plugin(PLGDIR,PlgArgs);
}

blogi::Blogi::~Blogi(){
    delete PlgArgs->auth;
    delete PlgArgs->session;
    delete PlgArgs->database;

    delete PlgArgs;
    delete BlogiPlg;
}

void blogi::Blogi::loginPage(netplus::con*curcon,libhttppp::HttpRequest *curreq){
    libhttppp::HTTPException excep;
    std::string sessid;
    if(PlgArgs->auth->isLoggedIn(curreq,sessid)){
        libhttppp::HTTPException err;
        err[libhttppp::HTTPException::Error] << "you already authenticated please logoff before you login again!";
        throw err;
    }
    libhtmlpp::HtmlString condat;
    condat << "<div id=\"content\">"
    << "<span>Login</span>"
    << "<form action=\""<< BUILDURL(login) << "\" method=\"post\">"
    << "username:<br> <input type=\"text\" name=\"username\" value=\"\"><br>"
    << "password:<br> <input type=\"password\" name=\"password\" value=\"\"><br>"
    << "<button type=\"submit\">Submit</button>"
    << "</form>"
    << "</div>";

    libhttppp::HttpForm curform;
    curform.parse(curreq);

    const char *username=nullptr;
    const char *password=nullptr;

    if (curform.getUrlcodedFormData()) {
        for (libhttppp::HttpForm::UrlcodedFormData* cururlform = curform.getUrlcodedFormData(); cururlform;
             cururlform = cururlform->nextUrlcodedFormData()) {
            if (strcmp(cururlform->getKey(), "username") == 0) {
                username = cururlform->getValue();
            }
            else if (strcmp(cururlform->getKey(), "password") == 0) {
                password = cururlform->getValue();
            }
        }
    }

    std::string out;

    if (!username || !password) {
        PlgArgs->theme->RenderSite(out,curreq->getRequestURL(),condat,false);
        libhttppp::HttpResponse curres;
        curres.setState(HTTP200);
        curres.setVersion(HTTPVERSION(1.1));
        curres.setContentType("text/html");
        curres.send(curcon,out.c_str(),out.length());
        return;
    }

    std::string sid;

    if(PlgArgs->auth->login(username,password,sid)){
        const char *sessid = PlgArgs->session->createSession(sid.c_str());
        PlgArgs->session->addSessionData(sessid,"sid",sid.c_str(),sid.length());
        PlgArgs->session->addSessionData(sessid,"username",username, strlen(username));
        libhttppp::HttpResponse curres;
        libhttppp::HttpCookie cookie;
        cookie.setcookie(&curres, "sessionid", sessid);
        curres.setState(HTTP307);
        curres.setVersion(HTTPVERSION(1.1));
        *curres.setData("Location") << STARTPAGE;
        curres.setContentType("text/html");
        curres.send(curcon, nullptr, 0);
    }else{
        libhttppp::HTTPException err;
        err[libhttppp::HTTPException::Error] << "Login Failed!";
        throw err;
    }
}

void blogi::Blogi::logoutPage(netplus::con *curcon,libhttppp::HttpRequest *curreq){
    const char *host;
    for(libhttppp::HttpHeader::HeaderData *preq = curreq->getfirstHeaderData(); preq; preq=curreq->nextHeaderData(preq)){
        if(strncmp(curreq->getKey(preq),"Host",4)==0)
            host=curreq->getValue(preq);
    }
    libhttppp::HttpResponse curres;
    libhttppp::HttpCookie cookie;
    cookie.setcookie(&curres,"sessionid","empty");
    curres.setState(HTTP307);
    curres.setVersion(HTTPVERSION(1.1));
    *curres.setData("Location") << STARTPAGE;
    curres.setContentType("text/html");
    curres.send(curcon,nullptr,0);
}

void blogi::Blogi::RequestEvent(netplus::con *curcon){
    libhttppp::HttpRequest req;
    req.parse(curcon);
    try{

        /*blogi internal pages and redirections*/
        if(strcmp(req.getRequestURL(),"/")==0 || strcmp(req.getRequestURL(),PREFIX)==0){
            libhttppp::HttpResponse curres;
            curres.setState(HTTP307);
            curres.setVersion(HTTPVERSION(1.1));
            *curres.setData("Location") << BUILDURL(content/tag);
            curres.setContentType("text/html");
            curres.send(curcon, nullptr, 0);
            return;
        }else if(strncmp(req.getRequestURL(),BUILDURL(logout),strlen(BUILDURL(logout)))==0){
            logoutPage(curcon,&req);
            return;
        }else if(strncmp(req.getRequestURL(),BUILDURL(login),strlen(BUILDURL(login)))==0){
            loginPage(curcon,&req);
            return;
        }


        if(!PlgArgs->theme->Controller(curcon,&req)){
            for(blogi::Plugin::PluginData *curplg=BlogiPlg->getFirstPlugin(); curplg; curplg=curplg->getNextPlg()){
                PluginApi *api=curplg->getInstace();
                std::string url=PREFIX;
                url+="/";
                url+=api->getName();
                if(strncmp(req.getRequestURL(),url.c_str(),url.length())==0){
                    if(api->Controller(curcon,&req))
                        return;
                }
            }

            std::string output;
            libhtmlpp::HtmlString err;
            err << "<!DOCTYPE html><html><body style=\"color:rgb(238, 238, 238); background:rgb(35, 38, 39);\"><span>"
            << "Seite oder Inhalt nicht gefudnen"
            << "</span><br/><a style=\"text-decoration: none; color: rgb(58,212, 58);\" href=\""
            <<  STARTPAGE
            << "\" >Zur&uuml;ck zur Startseite</a></html></body>";
            libhtmlpp::print(err.parse(),nullptr,output);
            libhttppp::HttpResponse resp;
            resp.setVersion(HTTPVERSION(1.1));
            resp.setContentType(HTTP404);
            resp.setContentType("text/html");
            resp.send(curcon,output.c_str(),output.length());

        }
    }catch(libhttppp::HTTPException &e){
        std::string output;
        libhtmlpp::HtmlString err;
        err << "<!DOCTYPE html><html><body style=\"color:rgb(238, 238, 238); background:rgb(35, 38, 39);\"><span>"
        << e.what()
        << "</span><br/><a style=\"text-decoration: none; color: rgb(58,212, 58);\" href=\""
        <<  STARTPAGE
        << "\" >Zur&uuml;ck zur Startseite</a></html></body>";
        libhtmlpp::print(err.parse(),nullptr,output);
        libhttppp::HttpResponse resp;
        resp.setVersion(HTTPVERSION(1.1));
        resp.setContentType(HTTP500);
        resp.setContentType("text/html");
        resp.send(curcon,output.c_str(),output.length());
    }
}


class HttpConD : public libhttppp::HttpD {
public:
    HttpConD(int argc, char** argv) : HttpD(argc,argv){
        libhttppp::HTTPException httpexception;
        try {
            blogi::Session session;
            blogi::Blogi blg(getServerSocket());
            blg.runEventloop();
        }catch(libhttppp::HTTPException &e){
            std::cout << e.what() << std::endl;
        }
    };
private:
};

int main(int argc, char** argv){
    signal(SIGPIPE, SIG_IGN);
    HttpConD(argc,argv);
}


