/*******************************************************************************
 * Copyright (c) 2023, Jan Koester jan.koester@gmx.net
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * Neither the name of the <organization> nor the
 *      names of its contributors may be used to endorse or promote products
 *      derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *******************************************************************************/

#include <yaml.h>

#include "conf.h"

blogi::ConfigValue::ConfigValue(){
    _firstValue=nullptr;
    _nextValue=nullptr;
    _Index=0;
}

blogi::ConfigValue::~ConfigValue(){
    delete _nextValue;
}

const char * blogi::ConfigValue::operator[](int idx){
    for(ConfigValue *cval=_firstValue; cval; cval=cval->_nextValue){
        if(cval->_Index==idx)
            return _Value.c_str();
    }
    return nullptr;
}

blogi::Config::Config(){
    firstData=nullptr;
    lastData=nullptr;
}

blogi::Config::~Config(){
    delete firstData;
}

blogi::ConfigValue & blogi::Config::Value(const char* key){
    for(ConfigData *cdat=firstData; cdat; cdat=cdat->nextData){
        if(cdat->Key==key){
            return cdat->Value;
        }
    }
    throw "Config: key not found";
}

blogi::Config::ConfigData & blogi::Config::Child(const char* key){
    for(ConfigData *cdat=firstData; cdat; cdat=cdat->nextData){
        if(cdat->Key==key){
            return *cdat->childData;
        }
    }
    throw "Config: key not found";
}

blogi::Config::ConfigData::ConfigData(){
    childData=nullptr;
    nextData=nullptr;
}

blogi::Config::ConfigData::~ConfigData(){
    delete childData;
    delete nextData;
}

size_t blogi::Config::ConfigData::getElements(){
    return Elements;
}


void blogi::Yaml::loadConfig(const char* path){
    FILE *fh = fopen(path,"r");

    yaml_parser_t parser;
    yaml_token_t  token;

     /* Initialize parser */
    if(!yaml_parser_initialize(&parser))
        throw "Failed to initialize parser!";
    if(fh == nullptr)
        throw "Failed to open file!";

    /* Set input file */
    yaml_parser_set_input_file(&parser, fh);

    /* BEGIN new code */
    do {
        yaml_parser_scan(&parser, &token);
        switch(token.type) {
            /* Token types (read before actual token) */

            case YAML_VALUE_TOKEN:{
                if(firstData){
                    lastData->nextData= new ConfigData;
                    lastData=lastData->nextData;
                }else{
                    firstData= new ConfigData;
                    lastData=firstData;
                }
            }break;
            default:
                printf("Got token of type %d\n", token.type);
    }
    if(token.type != YAML_STREAM_END_TOKEN)
        yaml_token_delete(&token);
    } while(token.type != YAML_STREAM_END_TOKEN);
    yaml_token_delete(&token);
    /* END new code */

    /* Cleanup */
    yaml_parser_delete(&parser);
    fclose(fh);



}

