/*******************************************************************************
 * Copyright (c) 2023, Jan Koester jan.koester@gmx.net
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * Neither the name of the <organization> nor the
 *      names of its contributors may be used to endorse or promote products
 *      derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *******************************************************************************/

#include <string>
#include <stdio.h>

#pragma once

namespace blogi {
    class ConfigValue{
    public:
        const char *operator[](int idx);
    protected:
        ConfigValue();
        ~ConfigValue();
        std::string   _Value;
        size_t        _Index;
        ConfigValue  *_nextValue;
        ConfigValue  *_firstValue;
        friend class Config;
    };

    class Config {
    public:
        Config();
        ~Config();

        class ConfigData{
        public:
            size_t getElements();
        protected:
            ConfigData();
            ~ConfigData();

            std::string  Key;
            size_t       Elements;
            ConfigValue  Value;
            ConfigData  *childData;
            ConfigData  *nextData;
            friend class Config;
            friend class Yaml;
        };

        ConfigValue &Value(const char* key);
        ConfigData  &Child(const char* key);

        virtual void loadConfig(const char *path)=0;

    protected:
        ConfigData *firstData;
        ConfigData *lastData;
    };

    class Yaml : public Config {
    public:
        void loadConfig(const char *path);
    };
};


#define PREFIX "/blog"

#define POSTGRES_DB      "host=pg-database.tuxist.de dbname=blog user=blog password=blabla sslmode=require"

#define LDAP_HOST        "ldap://10.1.2.200:389"
#define LDAP_BASEDN      "CN=Users,DC=intranet,DC=tuxist,DC=de"
#define LDAP_LOGINFILTER "(&(&(|(objectclass=person))(|(|(memberof=CN=blogi-users,CN=Users,DC=intranet,DC=tuxist,DC=de))))(samaccountname=%s))"

#define BUILDURL(V) (PREFIX"/"#V)

#define TEMPLATEPATH     "/usr/share/blogi/template"
#define TEMPLATENAME     "tuxist"
#define PLGDIR           "/usr/local/lib/blogi/plugins"
#define STARTPAGE        BUILDURL(content/index)
