/*******************************************************************************
 * Copyright (c) 2021, Jan Koester jan.koester@gmx.net
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * Neither the name of the <organization> nor the
 *      names of its contributors may be used to endorse or promote products
 *      derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *******************************************************************************/

#include <iostream>
#include <fstream>
#include <compare>

#include <dirent.h>
#include <dlfcn.h>
#include <unistd.h>
#include <sys/stat.h>

#include <httppp/exception.h>

#include <htmlpp/exception.h>

#include "theme.h"
#include "conf.h"

blogi::Template::Template(blogi::TemplateConfig& config){
    _Config=config;
    std::string idxpath = _Config.Path;
    idxpath += "/";
    idxpath += _Config.Theme;

    std::string htmlfile=idxpath;
    htmlfile.append("/index.html");

    _BlogIndex=_BlogPage.loadFile(htmlfile.c_str());

    DIR *directory = opendir(std::string(idxpath).append("/public").c_str());

    if (directory != nullptr) {
        struct dirent *direntStruct;
        while((direntStruct = readdir(directory))) {
            std::string buf;
            char tmp[BLOCKSIZE];
            std::ifstream fs;
            std::string fpath=std::string(idxpath).append("/public");
            fpath.append("/");
            fpath.append(direntStruct->d_name);

            struct stat curstat;
            stat(fpath.c_str(),&curstat);

            if(S_ISREG(curstat.st_mode)!=0){
                try{
                    fs.open(fpath);
                }catch(std::exception &e){
                    continue;
                }

                while (fs.good()) {
                    fs.read(tmp,BLOCKSIZE);
                    buf.append(tmp,fs.gcount());
                }
                _PublicFiles.insert(std::pair<std::string,std::string>(std::string("/theme/public/").append(direntStruct->d_name),buf));

                fs.close();
            }
        }
        closedir(directory);
    }

}

blogi::Template::~Template(){

}

bool blogi::Template::Controller(netplus::con *curcon,libhttppp::HttpRequest *req){
    std::string publicf = req->getRequestURL();
    if(publicf.length() >strlen(PREFIX) && publicf.compare(strlen(PREFIX),13,"/theme/public/",13)==0){
        for(auto curfile=_PublicFiles.begin(); curfile!=_PublicFiles.end(); curfile++){
            if(publicf.compare(strlen(PREFIX),curfile->first.length(),curfile->first,0,curfile->first.length())==0){
                libhttppp::HttpResponse resp;
                resp.setVersion(HTTPVERSION(1.1));
                resp.setState(HTTP200);

                std::string type=curfile->first.substr(curfile->first.rfind(".")+1,
                                                       curfile->first.length()-(curfile->first.rfind(".")+1));

                if(type=="png" || type=="jpg" || type=="webp"){
                    resp.setContentType(std::string("image/").append(type).c_str());
                }else if(type=="css" || type=="html"){
                    resp.setContentType(std::string("text/").append(type).c_str());
                }else if(type=="js"){
                    resp.setContentType("application/javascript");
                }else{
                    resp.setContentType("application/octet-stream");
                }

                resp.send(curcon,curfile->second.c_str(),curfile->second.length());
                return true;
            }
        }
    }
    return false;
}

void blogi::Template::RenderSite(std::string& output, const char* crrurl, libhtmlpp::HtmlString input,bool login){
    try{

        blogi::SQL sql;
        blogi::DBResult res;

        sql << "select url,name from navbar ORDER BY id";

        int n = _Config.TDatabase->exec(&sql,res);
        if(n<1){
            libhttppp::HTTPException excep;
            excep[libhttppp::HTTPException::Critical] << "No entries found for navbar";
            throw excep;
        }

        libhtmlpp::HtmlString buf;
        buf << "<div id=\"navbar\">"
        << "<ul style=\"display:inline; list-style-type: none;\">";

        for (int i = 0; i < n; i++) {
            buf << "<li style=\"padding:0 10px; float:left; border-right:solid 1px rgb(35, 38, 39);\" ";
            if(strncmp(crrurl,res[i][0],strlen(res[i][0]))==0)
                buf << "class=\"active\"";
            else
                buf << "class=\"inactive\"";
            buf << "><a href=\"" << res[i][0] << "\">" << res[i][1] << "</a></li>";
        }
        //<li class=\"active\" style=\"padding:1px 10px; float:left\"><a href=\"" << BUILDURL(index) <<"\">Blog</a></li>
        buf << "</ul></div>";
        std::string sessid;
        libhtmlpp::HtmlElement index;
        index = _BlogIndex;
        libhtmlpp::HtmlElement *header,*main, *footernav;
        header = index.getElementbyID("header");
        main =  index.getElementbyID("main");
        footernav = index.getElementbyID("footernav");
        header->appendChild(buf.parse());
        libhtmlpp::HtmlElement contentd("div");
        main->insertChild(input.parse());

        libhtmlpp::HtmlString footerancor;
        footerancor << "<a class=\"footer\" href=\"" << BUILDURL(staticpage/impressum) << "\" >Impressum</a>";
        if(!login){
            footerancor << "<a class=\"footer\" href=\"" << BUILDURL(login) << "\">Login</a> ";
        }else{
            footerancor << "<a class=\"footer\" href=\"" << BUILDURL(logout) << "\">Logout</a>"
            << "<a class=\"footer\" href=\"" << BUILDURL(settings) << "\">Settings</a>";
        }
        footernav->appendChild(footerancor.parse());
        libhtmlpp::print(&index,nullptr,output);
    }catch(libhtmlpp::HTMLException &e){
        libhttppp::HTTPException excep;
        excep[libhttppp::HTTPException::Error] << e.what();
        throw excep;
    }

}





